<?php
namespace PME\Controllers\Parametrizacao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class AtualizarComensaisCardController extends \PME\Services\TeknisaCrud {
	protected $crudService;
	protected $parameterBag;
	protected $connection;
	protected $armazenaResultado;
	protected $armazenaCentral;

    public function __construct($crudService, $parameterBag, $connection) {
        parent::__construct($crudService, $parameterBag); 
        $this->connection = $connection;
	}
	
	public function buscaComensaisUnidade(DTO\Request $request, DTO\Response $response) {
		try {
			$row = $request->getRow();
			$this->connection->beginTransaction();
		
			if(empty($row['UNIDADE'])) $unidade = 'T';
	        else $unidade = is_array($row['UNIDADE']) ? join("_", $row['UNIDADE']) : $row['UNIDADE'];
	        
	        if(empty($row['TIPOCARD'])) $tipocard = 'T';
	        else $tipocard = is_array($row['TIPOCARD']) ? join("_", $row['TIPOCARD']) : $row['TIPOCARD'];
	        
	        if(empty($row['SERVICO'])) $servico = 'T';
	        else $servico = is_array($row['SERVICO']) ? join("_", $row['SERVICO']) : $row['SERVICO'];
			
			$query = "SELECT CDFILCENTPME, CDTIPOCARD, CDFILIAL, CDSERVFILI, NVL(QTCOMPMESEG, 0) AS QTCOMPMESEG, NVL(QTCOMPMETER, 0) AS QTCOMPMETER, NVL(QTCOMPMEQUA, 0) AS QTCOMPMEQUA, NVL(QTCOMPMEQUI, 0) AS QTCOMPMEQUI, NVL(QTCOMPMESEX, 0) AS QTCOMPMESEX, NVL(QTCOMPMESAB, 0) AS QTCOMPMESAB, NVL(QTCOMPMEDOM, 0) AS QTCOMPMEDOM, NVL(QTCOMPMEFER, 0) AS QTCOMPMEFER, NVL(QTCLIPMESEG, 0) AS QTCLIPMESEG, NVL(QTCLIPMETER, 0) AS QTCLIPMETER, NVL(QTCLIPMEQUA, 0) AS QTCLIPMEQUA, NVL(QTCLIPMEQUI, 0) AS QTCLIPMEQUI, NVL(QTCLIPMESEX, 0) AS QTCLIPMESEX, NVL(QTCLIPMESAB, 0) AS QTCLIPMESAB, NVL(QTCLIPMEDOM, 0) AS QTCLIPMEDOM, NVL(QTCLIPMEFER, 0) AS QTCLIPMEFER, NVL(QTFUNPMESEG, 0) AS QTFUNPMESEG, NVL(QTFUNPMETER, 0) AS QTFUNPMETER, NVL(QTFUNPMEQUA, 0) AS QTFUNPMEQUA, NVL(QTFUNPMEQUI, 0) AS QTFUNPMEQUI, NVL(QTFUNPMESEX, 0) AS QTFUNPMESEX, NVL(QTFUNPMESAB, 0) AS QTFUNPMESAB, NVL(QTFUNPMEDOM, 0) AS QTFUNPMEDOM, NVL(QTFUNPMEFER, 0) AS QTFUNPMEFER, NVL(QTDOAPMESEG, 0) AS QTDOAPMESEG, NVL(QTDOAPMETER, 0) AS QTDOAPMETER, NVL(QTDOAPMEQUA, 0) AS QTDOAPMEQUA, NVL(QTDOAPMEQUI, 0) AS QTDOAPMEQUI, NVL(QTDOAPMESEX, 0) AS QTDOAPMESEX, NVL(QTDOAPMESAB, 0) AS QTDOAPMESAB, NVL(QTDOAPMEDOM, 0) AS QTDOAPMEDOM, NVL(QTDOAPMEFER, 0) AS QTDOAPMEFER FROM COMESERVTCAR CO WHERE (CO.CDFILCENTPME = :P_CDFILCENTPME) AND (CO.CDFILIAL <> :P_CDFILCENTPME) AND ((INSTR(:P_CDFILIAL, CO.CDFILIAL) > 0) OR (:P_CDFILIAL = 'T')) AND ((INSTR(:P_CDTIPOCARD, CO.CDTIPOCARD) > 0) OR (:P_CDTIPOCARD = 'T')) AND ((INSTR(:P_CDSERVFILI, CO.CDSERVFILI) > 0) OR (:P_CDSERVFILI = 'T'))";
			$params = array(
	            "P_CDFILCENTPME" => $row["CDFILCENTPME"],
	            "P_CDFILIAL"     => $unidade,
	            "P_CDTIPOCARD"   => $tipocard,
	            "P_CDSERVFILI"   => $servico
			);
			$stmt = $this->connection->prepare($query);
			$stmt->execute($params);
	        $result = $stmt->fetchAll();
	        $armazenaResultado = $result;
	
	        $this->updateComensaisSemana($request, $response, $armazenaResultado);
	        $this->updateComensaisFeriado($request, $response, $armazenaResultado);
	        
	        $this->connection->commit();
		} catch (\Exception $e) {
			$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function updateComensaisSemana(DTO\Request $request, DTO\Response $response, $armazenaResultado){
		try {
			$row = $request->getRow();
	        foreach($armazenaResultado as $pos){
				$data = array(
					"P_QTFUNPMEDOM"  => $pos['QTFUNPMEDOM'],
					"P_QTFUNPMESEG"  => $pos['QTFUNPMESEG'],
					"P_QTFUNPMETER"  => $pos['QTFUNPMETER'],
					"P_QTFUNPMEQUA"  => $pos['QTFUNPMEQUA'],
					"P_QTFUNPMEQUI"  => $pos['QTFUNPMEQUI'],
					"P_QTFUNPMESEX"  => $pos['QTFUNPMESEX'],
					"P_QTFUNPMESAB"  => $pos['QTFUNPMESAB'],
					"P_QTDOAPMEDOM"  => $pos['QTDOAPMEDOM'],
					"P_QTDOAPMESEG"  => $pos['QTDOAPMESEG'],
					"P_QTDOAPMETER"  => $pos['QTDOAPMETER'],
					"P_QTDOAPMEQUA"  => $pos['QTDOAPMEQUA'],
					"P_QTDOAPMEQUI"  => $pos['QTDOAPMEQUI'],
					"P_QTDOAPMESEX"  => $pos['QTDOAPMESEX'],
					"P_QTDOAPMESAB"  => $pos['QTDOAPMESAB'],
					"P_QTCLIPMEDOM"  => $pos['QTCLIPMEDOM'],
					"P_QTCLIPMESEG"  => $pos['QTCLIPMESEG'],
					"P_QTCLIPMETER"  => $pos['QTCLIPMETER'],
					"P_QTCLIPMEQUA"  => $pos['QTCLIPMEQUA'],
					"P_QTCLIPMEQUI"  => $pos['QTCLIPMEQUI'],
					"P_QTCLIPMESEX"  => $pos['QTCLIPMESEX'],
					"P_QTCLIPMESAB"  => $pos['QTCLIPMESAB'],
					"P_QTCOMPMEDOM"  => $pos['QTCOMPMEDOM'],
					"P_QTCOMPMESEG"  => $pos['QTCOMPMESEG'],
					"P_QTCOMPMETER"  => $pos['QTCOMPMETER'],
					"P_QTCOMPMEQUA"  => $pos['QTCOMPMEQUA'],
					"P_QTCOMPMEQUI"  => $pos['QTCOMPMEQUI'],
					"P_QTCOMPMESEX"  => $pos['QTCOMPMESEX'],
					"P_QTCOMPMESAB"  => $pos['QTCOMPMESAB'],
					"P_CDFILCENTPME" => $pos['CDFILCENTPME'],
					"P_CDFILIAL"     => $pos['CDFILIAL'],
					"P_CDSERVFILI"   => $pos['CDSERVFILI'],
					"P_DATAINI"      => $row['PERIODODE'],
					"P_DATAFIN"      => $row['PERIODOATE'],
					"P_CDTIPOCARD"   => $pos['CDTIPOCARD']
				);

			    $query = "UPDATE CARDIAPME SET QTFUNESTCPME = DECODE(TO_CHAR(DTCARDIAPME, 'D'), 1, :P_QTFUNPMEDOM, 2, :P_QTFUNPMESEG, 3, :P_QTFUNPMETER, 4, :P_QTFUNPMEQUA, 5, :P_QTFUNPMEQUI, 6, :P_QTFUNPMESEX, 7, :P_QTFUNPMESAB), QTDOAESTCPME = DECODE(TO_CHAR(DTCARDIAPME, 'D'), 1, :P_QTDOAPMEDOM, 2, :P_QTDOAPMESEG, 3, :P_QTDOAPMETER, 4, :P_QTDOAPMEQUA, 5, :P_QTDOAPMEQUI, 6, :P_QTDOAPMESEX, 7, :P_QTDOAPMESAB), QTCLIESTCPME = DECODE(TO_CHAR(DTCARDIAPME, 'D'), 1, :P_QTCLIPMEDOM, 2, :P_QTCLIPMESEG, 3, :P_QTCLIPMETER, 4, :P_QTCLIPMEQUA, 5, :P_QTCLIPMEQUI, 6, :P_QTCLIPMESEX, 7, :P_QTCLIPMESAB), QTCOMESTCPME = DECODE(TO_CHAR(DTCARDIAPME, 'D'), 1, :P_QTCOMPMEDOM, 2, :P_QTCOMPMESEG, 3, :P_QTCOMPMETER, 4, :P_QTCOMPMEQUA, 5, :P_QTCOMPMEQUI, 6, :P_QTCOMPMESEX, 7, :P_QTCOMPMESAB) WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (CDFILIAL = :P_CDFILIAL) AND (CDSERVFILI = :P_CDSERVFILI) AND (DTCARDIAPME BETWEEN :P_DATAINI AND :P_DATAFIN) AND (EXISTS ( SELECT CARDAPIOPME.NRCARDPME FROM CARDAPIOPME WHERE (CARDAPIOPME.CDFILCENTPME = CARDIAPME.CDFILCENTPME) AND (CARDAPIOPME.NRCARDPME = CARDIAPME.NRCARDPME) AND (DTCARDIAPME BETWEEN CARDAPIOPME.DTINICARDPME AND CARDAPIOPME.DTFINCARDPME) AND (CARDAPIOPME.CDTIPOCARD = :P_CDTIPOCARD) )) AND (NOT EXISTS ( SELECT DTFERIFILI FROM FERIFILI FF WHERE FF.DTFERIFILI = CARDIAPME.DTCARDIAPME AND FF.CDFILIAL = CARDIAPME.CDFILIAL))";
			    $stmt = $this->connection->prepare($query);
				$stmt->execute($data);
	        }
	    } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function updateComensaisFeriado(DTO\Request $request, DTO\Response $response, $armazenaResultado){
		try{
			$row = $request->getRow();
			foreach($armazenaResultado as $pos){
				$data = array(
					"P_QTFUNPMEFER"  => $pos['QTFUNPMEFER'],
					"P_QTDOAPMEFER"  => $pos['QTDOAPMEFER'],
					"P_QTCLIPMEFER"  => $pos['QTCLIPMEFER'],
					"P_QTCOMPMEFER"  => $pos['QTCOMPMEFER'],
					"P_CDFILCENTPME" => $pos['CDFILCENTPME'],
					"P_CDFILIAL"     => $pos['CDFILIAL'],
					"P_CDSERVFILI"   => $pos['CDSERVFILI'],
					"P_DATAINI"      => $row['PERIODODE'],
					"P_DATAFIN"      => $row['PERIODOATE'],
					"P_CDTIPOCARD"   => $pos['CDTIPOCARD']
				);
		    	$query = "UPDATE CARDIAPME SET QTFUNESTCPME = :P_QTFUNPMEFER, QTDOAESTCPME = :P_QTDOAPMEFER, QTCLIESTCPME = :P_QTCLIPMEFER, QTCOMESTCPME = :P_QTCOMPMEFER WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (CDFILIAL = :P_CDFILIAL)         AND (CDSERVFILI = :P_CDSERVFILI)     AND (DTCARDIAPME  BETWEEN :P_DATAINI AND :P_DATAFIN) AND (EXISTS (SELECT CARDAPIOPME.NRCARDPME FROM CARDAPIOPME WHERE (CARDAPIOPME.CDFILCENTPME = CARDIAPME.CDFILCENTPME) AND (CARDAPIOPME.NRCARDPME = CARDIAPME.NRCARDPME) AND (DTCARDIAPME BETWEEN CARDAPIOPME.DTINICARDPME AND CARDAPIOPME.DTFINCARDPME) AND (CARDAPIOPME.CDTIPOCARD = :P_CDTIPOCARD) ) )AND (EXISTS (SELECT DTFERIFILI FROM FERIFILI FF WHERE FF.DTFERIFILI = CARDIAPME.DTCARDIAPME AND FF.CDFILIAL = CARDIAPME.CDFILIAL))";
		    	$stmt = $this->connection->prepare($query);
				$stmt->execute($data);
			}
		} catch(\Exception $e){
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function buscaTotalCentral(DTO\Request $request, DTO\Response $response){
		try {
			$row = $request->getRow();
			$this->connection->beginTransaction();
        
	        if(empty($row['TIPOCARD'])) $tipocard = 'T';
	        else $tipocard = is_array($row['TIPOCARD']) ? join("_", $row['TIPOCARD']) : $row['TIPOCARD'];
	        
	        if(empty($row['SERVICO'])) $servico = 'T';
	        else $servico = is_array($row['SERVICO']) ? join("_", $row['SERVICO']) : $row['SERVICO'];
	        
		    $query = "SELECT CA.CDFILCENTPME, CA.CDSERVFILI, TO_DATE(TO_CHAR(CA.DTCARDIAPME, 'DD/MM/YYYY')) AS DTCARDIAPME, CP.CDTIPOCARD, SUM(CA.QTCOMESTCPME) AS QTCOMESTCPME, SUM(CA.QTCLIESTCPME) AS QTCLIESTCPME, SUM(CA.QTFUNESTCPME) AS QTFUNESTCPME, SUM(CA.QTDOAESTCPME) AS QTDOAESTCPME FROM CARDIAPME CA, CARDAPIOPME CP WHERE (CA.CDFILCENTPME = :P_CDFILCENTPME) AND (CP.CDFILCENTPME = CA.CDFILCENTPME) AND (CP.NRCARDPME = CA.NRCARDPME) AND ((INSTR(:P_CDTIPOCARD, CP.CDTIPOCARD) >0) OR (:P_CDTIPOCARD  = 'T')) AND (CA.CDFILIAL <> :P_CDFILCENTPME) AND ((INSTR(:P_CDSERVFILI, CA.CDSERVFILI) >0) OR (:P_CDSERVFILI  = 'T')) AND (CA.IDCARDESPEC <> 'S') AND (CA.DTCARDIAPME BETWEEN :P_DATAINI AND :P_DATAFIN) GROUP BY CA.CDFILCENTPME, CA.CDSERVFILI, CP.CDTIPOCARD, CA.DTCARDIAPME";
		    $params = array(
	            "P_CDFILCENTPME" => $row["CDFILCENTPME"],
	            "P_CDTIPOCARD"   => $tipocard,
	            "P_CDSERVFILI"   => $servico,
	            "P_DATAINI"      => $row['PERIODODE'],
	            "P_DATAFIN"      => $row['PERIODOATE']
	        );
	        $stmt = $this->connection->prepare($query);
			$stmt->execute($params);
	        $result = $stmt->fetchAll();
	        $armazenaCentral = $result;
	
	        $this->atulizaComensaisDia($request, $response, $armazenaCentral);
			
			$this->connection->commit();
		} catch (\Exception $e){
			$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function atulizaComensaisDia(DTO\Request $request, DTO\Response $response, $armazenaCentral){
		try {
			$row = $request->getRow();
	        foreach($armazenaCentral as $pos){
				$data = array(
					"P_QTFUNESTCPME" => $pos['QTFUNESTCPME'],
					"P_QTDOAESTCPME" => $pos['QTDOAESTCPME'],
					"P_QTCLIESTCPME" => $pos['QTCLIESTCPME'],
					"P_QTCOMESTCPME" => $pos['QTCOMESTCPME'],
					"P_CDFILCENTPME" => $pos['CDFILCENTPME'],
					"P_CDFILCENTPME" => $pos['CDFILCENTPME'],
					"P_CDSERVFILI"   => $pos['CDSERVFILI'],
					"P_DTCARDIAPME"  => substr($pos['DTCARDIAPME'], 0, 10),
					"P_CDTIPOCARD"   => $pos['CDTIPOCARD']	
				);
	    		$query = "UPDATE CARDIAPME SET QTFUNESTCPME = :P_QTFUNESTCPME, QTDOAESTCPME = :P_QTDOAESTCPME, QTCLIESTCPME = :P_QTCLIESTCPME, QTCOMESTCPME = :P_QTCOMESTCPME WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (CDFILIAL = :P_CDFILCENTPME)         AND (CDSERVFILI = :P_CDSERVFILI)     AND (TO_DATE(TO_CHAR(DTCARDIAPME, 'DD/MM/YYYY')) = :P_DTCARDIAPME)   AND (EXISTS (SELECT CARDAPIOPME.NRCARDPME FROM CARDAPIOPME WHERE (CARDAPIOPME.CDFILCENTPME = CARDIAPME.CDFILCENTPME) AND (CARDAPIOPME.NRCARDPME = CARDIAPME.NRCARDPME) AND (DTCARDIAPME BETWEEN CARDAPIOPME.DTINICARDPME AND CARDAPIOPME.DTFINCARDPME) AND (CARDAPIOPME.CDTIPOCARD = :P_CDTIPOCARD) ) )AND (NOT EXISTS (SELECT DTFERIFILI FROM FERIFILI FF WHERE FF.DTFERIFILI = CARDIAPME.DTCARDIAPME AND FF.CDFILIAL = CARDIAPME.CDFILIAL ) )";
			    $stmt = $this->connection->prepare($query);
				$stmt->execute($data);
	        }
	    } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
}