<?php
namespace Folha\Logic\ESocial\Eventos\Data;
use Folha\Repositories\GpeEsocialRegistro;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-1250 - Aquisição de Produção Rural.
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author felipesantos
 **/
class S1250
{

    /**
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1250, Aquisição de Produção Rural.
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $idRetificacao           Identificador de Retificação dos arquivos ou não.
     * @param $nrRecibo                Número do Recibo do primeiro envio em caso de Retificação.
     * @param $idApuracao              Identificador da apuração Mensal ou Anual.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author felipesantos
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo, $indApuracao)
    {
        $S1250 = GpeEsocialRegistro::dataSetEsocial_S1250(
            $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
            $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo, $indApuracao
        );

        foreach ($S1250 as &$row) {
            $tiposAquis = GpeEsocialRegistro::s1250_tipoAquisicao($row['CDFILIRECE'], $inicioPeriodo, $fimPeriodo);

            $tpAquis = null;
            foreach($tiposAquis as &$tipo) {
                // Busca os produtores referentes ao tipo de aquisição
                $prods = GpeEsocialRegistro::s1250_ideProdutor($row['CDFILIRECE'], $inicioPeriodo, $fimPeriodo, $tipo['IDTPIJURFORN']);

                $produtor = null;
                foreach($prods as &$prod) {
                    // Busca as notas referentes ao produtor
                    $notas = GpeEsocialRegistro::s1250_Nfs($prod['CDFILIRECE'], $prod['CDFORNECED'], $inicioPeriodo, $fimPeriodo);

                    $notasfiscais = null;
                    foreach ($notas as &$notafiscal) {
                        $notasfiscais[] =
                            array(
                                "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NFS_SERIE" => $notafiscal['SERIE'],
                                "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NFS_NRDOCTO" => $notafiscal['NUMDOCTO'],
                                "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NFS_DTEMISNF" => $notafiscal['DTEMISSAONF'],
                                "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NFS_VLRBRUTO" => $notafiscal['NFVLRBRUTO'],
                                "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NFS_VRCPDESCPR" => $notafiscal['NFVLRCONTRIBDESCPR'],
                                "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NFS_VRRATDESCPR" => $notafiscal['NFVLRGILRATDESCPR'],
                                "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NFS_VRSENARDESC" => $notafiscal['NFVLRSENARDESCPR']);
                    }

                    $produtor[] =
                        array(
                            "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_TPINSCPROD" => $prod['TPINSCPROD'],
                            "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NRINSCPROD" => $prod['NRINSCPROD'],
                            "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_VLRBRUTO" => $prod['VLRBRUTO'],
                            "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_VRCPDESCPR" => $prod['VRCPDESCPR'],
                            "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_VRRATDESCPR" => $prod['VRRATDESCPR'],
                            "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_VRSENARDESC" => $prod['VRSENARDESC'],
                            "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR_NFS" => $notasfiscais
                        );

                }

                $tpAquis[] =
                    array(
                        "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_INDAQUIS" => $tipo['INDAQUIS'],
                        "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_VLRTOTAQUIS" => $tipo['VLRTOTAQUIS'],
                        "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS_IDEPRODUTOR" => $produtor
                    );
            }

            $estabAdquir[] =
                array(
                    "_EVTAQPROD_INFOAQUISPROD_IDEESTABADQUIR_TPAQUIS" => $tpAquis
                );
        }

        return $S1250;
    }

}